@@ -4,6 +4,8 @@ pub fn fixlengths_cmd() -> Command {
     Command::new("fixlengths").args([
         arg!(--length),
         arg!(--insert),
+        arg!(--quote),
+        arg!(--escape),
         arg!(--output),
         arg!(--delimiter),
     ])
