@@ -46,6 +46,13 @@ join options:
                            corresponding row in the first data set. When no
                            corresponding row exists, it is padded out with
                            empty fields. (This is the reverse of 'outer left'.)
+    --right-anti           This returns only the rows in the second CSV data set
+                           that do not have a corresponding row in the first
+                           data set. The output schema is the same as the
+                           second dataset.
+    --right-semi           This returns only the rows in the second CSV data set
+                           that have a corresponding row in the first data set.
+                           The output schema is the same as the second data set.
     --full                 Do a 'full outer' join. This returns all rows in
                            both data sets with matching records joined. If
                            there is no match, the missing side will be padded
@@ -60,6 +67,12 @@ join options:
                            Otherwise, empty fields are completely ignored.
                            (In fact, any row that has an empty field in the
                            key specified is ignored.)
+    --keys-output <file>   Write successfully joined keys to <file>.
+                           This means that the keys are written to the output
+                           file when a match is found, with the exception of
+                           anti joins, where keys are written when NO match
+                           is found.
+                           Cross joins do not write keys.
 
 Common options:
     -h, --help             Display this message
@@ -71,7 +84,7 @@ Common options:
                            Must be a single character. (default: ,)
 "#;
 
-use std::{collections::hash_map::Entry, fmt, io, iter::repeat, str};
+use std::{collections::hash_map::Entry, fmt, io, iter::repeat, mem::swap, str};
 
 use ahash::AHashMap;
 use byteorder::{BigEndian, WriteBytesExt};
@@ -96,6 +109,8 @@ struct Args {
     flag_left_anti:   bool,
     flag_left_semi:   bool,
     flag_right:       bool,
+    flag_right_anti:  bool,
+    flag_right_semi:  bool,
     flag_full:        bool,
     flag_cross:       bool,
     flag_output:      Option<String>,
@@ -103,6 +118,7 @@ struct Args {
     flag_ignore_case: bool,
     flag_nulls:       bool,
     flag_delimiter:   Option<Delimiter>,
+    flag_keys_output: Option<String>,
 }
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
@@ -113,37 +129,64 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         args.flag_left_anti,
         args.flag_left_semi,
         args.flag_right,
+        args.flag_right_anti,
+        args.flag_right_semi,
         args.flag_full,
         args.flag_cross,
     ) {
-        (true, false, false, false, false, false) => {
+        // default inner join
+        (false, false, false, false, false, false, false, false) => {
+            state.write_headers()?;
+            state.inner_join()
+        },
+        // left join
+        (true, false, false, false, false, false, false, false) => {
             state.write_headers()?;
             state.outer_join(false)
         },
-        (false, true, false, false, false, false) => {
+        // left anti join
+        (false, true, false, false, false, false, false, false) => {
             state.write_headers1()?;
             state.left_join(true)
         },
-        (false, false, true, false, false, false) => {
+        // left semi join
+        (false, false, true, false, false, false, false, false) => {
             state.write_headers1()?;
             state.left_join(false)
         },
-        (false, false, false, true, false, false) => {
+        // right join
+        (false, false, false, true, false, false, false, false) => {
             state.write_headers()?;
             state.outer_join(true)
         },
-        (false, false, false, false, true, false) => {
+        // right anti join
+        // swap left and right data sets and run left anti join
+        (false, false, false, false, true, false, false, false) => {
+            state.write_headers1()?;
+            let mut swapped_join = state;
+            swap(&mut swapped_join.rdr1, &mut swapped_join.rdr2);
+            swap(&mut swapped_join.sel1, &mut swapped_join.sel2);
+            swapped_join.left_join(true)
+        },
+        // right semi join
+        // swap left and right data sets and run left semi join
+        (false, false, false, false, false, true, false, false) => {
+            state.write_headers1()?;
+            let mut swapped_join = state;
+            swap(&mut swapped_join.rdr1, &mut swapped_join.rdr2);
+            swap(&mut swapped_join.sel1, &mut swapped_join.sel2);
+            swapped_join.left_join(false)
+        },
+        // full outer join
+        (false, false, false, false, false, false, true, false) => {
             state.write_headers()?;
             state.full_outer_join()
         },
-        (false, false, false, false, false, true) => {
+        // cross join
+        (false, false, false, false, false, false, false, true) => {
             state.write_headers()?;
             state.cross_join()
         },
-        (false, false, false, false, false, false) => {
-            state.write_headers()?;
-            state.inner_join()
-        },
         _ => fail_incorrectusage_clierror!("Please pick exactly one join operation."),
     }
 }
@@ -157,6 +200,7 @@ struct IoState<R, W: io::Write> {
     no_headers: bool,
     casei:      bool,
     nulls:      bool,
+    keys_wtr:   KeysWriter,
 }
 
 impl<R: io::Read + io::Seek, W: io::Write> IoState<R, W> {
@@ -181,93 +225,132 @@ impl<R: io::Read + io::Seek, W: io::Write> IoState<R, W> {
         let mut scratch = csv::ByteRecord::new();
         let mut validx = ValueIndex::new(self.rdr2, &self.sel2, self.casei, self.nulls)?;
         let mut row = csv::ByteRecord::new();
-        let mut key;
+        let mut key: Vec<ByteString>;
+        let mut output = csv::ByteRecord::new();
+
         while self.rdr1.read_byte_record(&mut row)? {
             key = get_row_key(&self.sel1, &row, self.casei);
             if let Some(rows) = validx.values.get(&key) {
+                self.keys_wtr.write_key(&key)?;
+
                 for &rowi in rows {
                     validx.idx.seek(rowi as u64)?;
 
                     validx.idx.read_byte_record(&mut scratch)?;
-                    let combined = row.iter().chain(scratch.iter());
-                    self.wtr.write_record(combined)?;
+
+                    output.clear();
+                    output.extend(&row);
+                    output.extend(&scratch);
+                    self.wtr.write_record(&output)?;
                 }
             }
         }
-        Ok(self.wtr.flush()?)
+        self.wtr.flush()?;
+        self.keys_wtr.flush()?;
+        Ok(())
     }
 
     fn outer_join(mut self, right: bool) -> CliResult<()> {
         if right {
-            ::std::mem::swap(&mut self.rdr1, &mut self.rdr2);
-            ::std::mem::swap(&mut self.sel1, &mut self.sel2);
+            swap(&mut self.rdr1, &mut self.rdr2);
+            swap(&mut self.sel1, &mut self.sel2);
         }
 
         let mut scratch = csv::ByteRecord::new();
         let (_, pad2) = self.get_padding()?;
         let mut validx = ValueIndex::new(self.rdr2, &self.sel2, self.casei, self.nulls)?;
         let mut row = csv::ByteRecord::new();
-        let mut key;
+        let mut key: Vec<ByteString>;
+        let mut output = csv::ByteRecord::new();
+
         while self.rdr1.read_byte_record(&mut row)? {
             key = get_row_key(&self.sel1, &row, self.casei);
             if let Some(rows) = validx.values.get(&key) {
+                self.keys_wtr.write_key(&key)?;
+
                 for &rowi in rows {
                     validx.idx.seek(rowi as u64)?;
-                    let row1 = row.iter();
                     validx.idx.read_byte_record(&mut scratch)?;
+                    output.clear();
                     if right {
-                        self.wtr.write_record(scratch.iter().chain(row1))?;
+                        output.extend(&scratch);
+                        output.extend(&row);
                     } else {
-                        self.wtr.write_record(row1.chain(&scratch))?;
+                        output.extend(&row);
+                        output.extend(&scratch);
                     }
+                    self.wtr.write_record(&output)?;
                 }
-            } else if right {
-                self.wtr.write_record(pad2.iter().chain(&row))?;
             } else {
-                self.wtr.write_record(row.iter().chain(&pad2))?;
+                output.clear();
+                if right {
+                    output.extend(&pad2);
+                    output.extend(&row);
+                } else {
+                    output.extend(&row);
+                    output.extend(&pad2);
+                }
+                self.wtr.write_record(&output)?;
             }
         }
-        Ok(self.wtr.flush()?)
+        self.wtr.flush()?;
+        self.keys_wtr.flush()?;
+        Ok(())
     }
 
     fn left_join(mut self, anti: bool) -> CliResult<()> {
         let validx = ValueIndex::new(self.rdr2, &self.sel2, self.casei, self.nulls)?;
         let mut row = csv::ByteRecord::new();
-        let mut key;
+        let mut key: Vec<ByteString>;
+
         while self.rdr1.read_byte_record(&mut row)? {
             key = get_row_key(&self.sel1, &row, self.casei);
             if validx.values.get(&key).is_none() {
                 if anti {
+                    self.keys_wtr.write_key(&key)?;
                     self.wtr.write_record(&row)?;
                 }
             } else if !anti {
+                self.keys_wtr.write_key(&key)?;
                 self.wtr.write_record(&row)?;
             }
         }
-        Ok(self.wtr.flush()?)
+        self.wtr.flush()?;
+        self.keys_wtr.flush()?;
+        Ok(())
     }
 
     fn full_outer_join(mut self) -> CliResult<()> {
         let mut scratch = csv::ByteRecord::new();
         let (pad1, pad2) = self.get_padding()?;
         let mut validx = ValueIndex::new(self.rdr2, &self.sel2, self.casei, self.nulls)?;
+        let mut output = csv::ByteRecord::new();
 
         // Keep track of which rows we've written from rdr2.
         let mut rdr2_written: Vec<_> = repeat(false).take(validx.num_rows).collect();
         let mut row1 = csv::ByteRecord::new();
-        let mut key;
+        let mut key: Vec<ByteString>;
+
         while self.rdr1.read_byte_record(&mut row1)? {
             key = get_row_key(&self.sel1, &row1, self.casei);
             if let Some(rows) = validx.values.get(&key) {
+                self.keys_wtr.write_key(&key)?;
+
                 for &rowi in rows {
                     rdr2_written[rowi] = true;
 
                     validx.idx.seek(rowi as u64)?;
                     validx.idx.read_byte_record(&mut scratch)?;
-                    self.wtr.write_record(row1.iter().chain(&scratch))?;
+                    output.clear();
+                    output.extend(&row1);
+                    output.extend(&scratch);
+                    self.wtr.write_record(&output)?;
                 }
             } else {
-                self.wtr.write_record(row1.iter().chain(&pad2))?;
+                output.clear();
+                output.extend(&row1);
+                output.extend(&pad2);
+                self.wtr.write_record(&output)?;
             }
         }
 
@@ -277,10 +360,15 @@ impl<R: io::Read + io::Seek, W: io::Write> IoState<R, W> {
             if !written {
                 validx.idx.seek(i as u64)?;
                 validx.idx.read_byte_record(&mut scratch)?;
-                self.wtr.write_record(pad1.iter().chain(&scratch))?;
+                output.clear();
+                output.extend(&pad1);
+                output.extend(&scratch);
+                self.wtr.write_record(&output)?;
             }
         }
-        Ok(self.wtr.flush()?)
+        self.wtr.flush()?;
+        self.keys_wtr.flush()?;
+        Ok(())
     }
 
     fn cross_join(mut self) -> CliResult<()> {
@@ -288,6 +376,7 @@ impl<R: io::Read + io::Seek, W: io::Write> IoState<R, W> {
         pos.set_byte(0);
         let mut row2 = csv::ByteRecord::new();
         let mut row1 = csv::ByteRecord::new();
+        let mut output = csv::ByteRecord::new();
         let rdr2_has_headers = self.rdr2.has_headers();
         while self.rdr1.read_byte_record(&mut row1)? {
             self.rdr2.seek(pos.clone())?;
@@ -297,7 +386,10 @@ impl<R: io::Read + io::Seek, W: io::Write> IoState<R, W> {
                 self.rdr2.read_byte_record(&mut row2)?;
             }
             while self.rdr2.read_byte_record(&mut row2)? {
-                self.wtr.write_record(row1.iter().chain(&row2))?;
+                output.clear();
+                output.extend(&row1);
+                output.extend(&row2);
+                self.wtr.write_record(&output)?;
             }
         }
         Ok(self.wtr.flush()?)
@@ -326,9 +418,22 @@ impl Args {
             .no_headers(self.flag_no_headers)
             .select(self.arg_columns2.clone());
 
-        let mut rdr1 = rconf1.reader_file_stdin()?;
-        let mut rdr2 = rconf2.reader_file_stdin()?;
+        let mut rdr1 = match rconf1.reader_file_stdin() {
+            Ok(rdr1) => rdr1,
+            Err(e) => return fail_clierror!("Failed to read input1: {e}"),
+        };
+        let mut rdr2 = match rconf2.reader_file_stdin() {
+            Ok(rdr2) => rdr2,
+            Err(e) => return fail_clierror!("Failed to read input2: {e}"),
+        };
         let (sel1, sel2) = self.get_selections(&rconf1, &mut rdr1, &rconf2, &mut rdr2)?;
+
+        let keys_wtr = if self.flag_cross {
+            KeysWriter::new(None)?
+        } else {
+            KeysWriter::new(self.flag_keys_output.as_ref())?
+        };
+
         Ok(IoState {
             wtr: Config::new(self.flag_output.as_ref()).writer()?,
             rdr1,
@@ -338,6 +443,7 @@ impl Args {
             no_headers: rconf1.no_headers,
             casei: self.flag_ignore_case,
             nulls: self.flag_nulls,
+            keys_wtr,
         })
     }
 
@@ -379,8 +485,8 @@ impl<R: io::Read + io::Seek> ValueIndex<R> {
         casei: bool,
         nulls: bool,
     ) -> CliResult<ValueIndex<R>> {
-        let mut val_idx = AHashMap::with_capacity(10000);
-        let mut row_idx = io::Cursor::new(Vec::with_capacity(8 * 10000));
+        let mut val_idx = AHashMap::with_capacity(20_000);
+        let mut row_idx = io::Cursor::new(Vec::with_capacity(8 * 20_000));
         let (mut rowi, mut count) = (0_usize, 0_usize);
 
         // This logic is kind of tricky. Basically, we want to include
@@ -458,3 +564,36 @@ impl<R> fmt::Debug for ValueIndex<R> {
 fn get_row_key(sel: &Selection, row: &csv::ByteRecord, casei: bool) -> Vec<ByteString> {
     sel.select(row).map(|v| util::transform(v, casei)).collect()
 }
+
+struct KeysWriter {
+    writer:  csv::Writer<Box<dyn io::Write>>,
+    enabled: bool,
+}
+
+impl KeysWriter {
+    fn new(keys_path: Option<&String>) -> CliResult<Self> {
+        let (writer, enabled) = if let Some(path) = keys_path {
+            (Config::new(Some(path)).writer()?, true)
+        } else {
+            let sink: Box<dyn io::Write> = Box::new(std::io::sink());
+            (csv::WriterBuilder::new().from_writer(sink), false)
+        };
+
+        Ok(Self { writer, enabled })
+    }
+
+    #[inline]
+    fn write_key(&mut self, key: &[ByteString]) -> CliResult<()> {
+        if self.enabled {
+            self.writer.write_record(key)?;
+        }
+        Ok(())
+    }
+
+    fn flush(&mut self) -> CliResult<()> {
+        if self.enabled {
+            self.writer.flush()?;
+        }
+        Ok(())
+    }
+}
