@@ -1,14 +1,12 @@
 static USAGE: &str = r#"
-Joins two sets of CSV data on the specified columns using the Pola.rs engine.
+Joins two sets of CSV data on the specified columns using the Polars engine.
 
 The default join operation is an 'inner' join. This corresponds to the
 intersection of rows on the keys specified.
 
 Unlike the join command, joinp can process files larger than RAM, is multithreaded,
-has join key validation, pre-join filtering, supports asof joins & its output columns
-can be coalesced (no duplicate columns).
-
-However, joinp doesn't have an --ignore-case option.
+has join key validation, a maintain row order option, pre-join filtering, supports
+asof joins and its output columns can be coalesced (no duplicate columns).
 
 Returns the shape of the join result (number of rows, number of columns) to stderr.
 
@@ -17,10 +15,12 @@ For examples, see https://github.com/dathere/qsv/blob/master/tests/test_joinp.rs
 Usage:
     qsv joinp [options] <columns1> <input1> <columns2> <input2>
     qsv joinp --cross [--validate <arg>] <input1> <input2> [--output <file>]
+    qsv joinp --non-equi <expr> <input1> <input2> [options] [--output <file>]
     qsv joinp --help
 
 joinp arguments:
-    Both <input1> and <input2> files need to have headers. Stdin is not supported.
+    Both <input1> aka <left> & <input2> aka <right> files need to have headers.
+    Stdin is not supported.
 
     The columns arguments specify the columns to join for each input. Columns are
     referenced by name. Specify multiple columns by separating them with a comma.
@@ -30,6 +30,10 @@ joinp arguments:
 
 joinp options:
     -i, --ignore-case      When set, joins are done case insensitively.
+-z, --ignore-leading-zeros  When set, joins are done ignoring leading zeros.
+                           Note that this is only applied to the join keys for
+                           both numeric and string columns. The output columns
+                           will not have leading zeros.
     --left                 Do a 'left outer' join. This returns all rows in
                            first CSV data set, including rows with no
                            corresponding row in the second data set. When no
@@ -47,6 +51,13 @@ joinp options:
                            corresponding row in the first data set. When no
                            corresponding row exists, it is padded out with
                            empty fields. (This is the reverse of 'outer left'.)
+    --right-anti           This returns only the rows in the second CSV data set
+                           that do not have a corresponding row in the first
+                           data set. The output schema is the same as the
+                           second dataset.
+    --right-semi           This returns only the rows in the second CSV data set
+                           that have a corresponding row in the first data set.
+                           The output schema is the same as the second data set.
     --full                 Do a 'full outer' join. This returns all rows in
                            both data sets with matching records joined. If
                            there is no match, the missing side will be padded
@@ -57,6 +68,17 @@ joinp options:
                            equal to N * M, where N and M correspond to the
                            number of rows in the given data sets, respectively.
                            The columns1 and columns2 arguments are ignored.
+    --non-equi <expr>      Do a non-equi join. The given expression is evaluated
+                           for each row in the left dataset and can refer to columns
+                           in the left and right dataset. If the expression evaluates
+                           to true, the row is joined with the corresponding row in
+                           the right dataset.
+                           The expression is a valid Polars SQL where clause, with each
+                           column name followed by "_left" or "_right" suffixes to indicate
+                           which data set the column belongs to.
+                           (e.g. "salary_left >= min_salary_right AND \
+                                  salary_left <= max_salary_right AND \
+                                  experience_left >= min_exp_right")
 
     --coalesce             Force the join to coalesce columns with the same name.
                            For inner joins, this is not necessary as the join
@@ -76,7 +98,13 @@ joinp options:
                              onetoone - join keys are unique in both left & right data sets.
                            [default: none]
 
-                           JOIN OPTIONS:
+                            JOIN OPTIONS:
+    --maintain-order <arg>  Which row order to preserve, if any. Valid values are:
+                              none, left, right, left_right, right_left
+                            Do not rely on any observed ordering without explicitly
+                            setting this parameter. Not specifying any order can improve
+                            performance. Supported for inner, left, right and full joins.
+                            [default: none]
     --nulls                When set, joins will work on empty fields.
                            Otherwise, empty fields are completely ignored.
     --streaming            When set, the join will be done in a streaming fashion.
@@ -91,18 +119,23 @@ joinp options:
                            enabled when using asof joins.
     --infer-len <arg>      The number of rows to scan when inferring the schema of the CSV.
                            Set to 0 to do a full table scan (warning: very slow).
+                           Only used when --cache-schema is 0 or 1 and no cached schema exists or
+                           when --infer-len is 0.
                            [default: 10000]
-    --cache-schema         Create and cache Polars schema JSON files.
-                           If specified and the schema file/s do not exist, it will check if a
-                           stats cache is available. If so, it will use it to derive a Polars schema
-                           and save it. If there's no stats cache, it will infer the schema 
-                           using --infer-len and save the inferred schemas. 
-                           Each schema file will have the same file stem as the corresponding
-                           input file, with the extension ".pschema.json"
-                           (data.csv's Polars schema file will be data.pschema.json)
-                           If the file/s exists, it will load the schema instead of inferring it
-                           (ignoring --infer-len) and attempt to use it for each corresponding
-                           Polars "table" with the same file stem.
+    --cache-schema <arg>   Create and cache Polars schema JSON files.
+                           Ignored when --infer-len is 0.
+                           ‎ -2: treat all columns as String. A Polars schema file is created & cached.
+                           ‎ -1: treat all columns as String. No Polars schema file is created.
+                             0: do not cache Polars schema. Uses --infer-len to infer schema.
+                             1: cache Polars schema with the following behavior:
+                                - If schema file exists and is newer than input: use cached schema
+                                - If schema file missing/outdated and stats cache exists: 
+                                  derive schema from stats and cache it
+                                - If no schema or stats cache: infer schema using --infer-len 
+                                  and cache the result
+                                Schema files use the same name as input with .pschema.json extension
+                                (e.g., data.csv -> data.pschema.json)
+                           [default: 0]
     --low-memory           Use low memory mode when parsing CSVs. This will use less memory
                            but will be slower. It will also process the join in streaming mode.
                            Only use this when you get out of memory errors.
@@ -173,9 +206,9 @@ joinp options:
 
                              OUTPUT FORMAT OPTIONS:
    --sql-filter <SQL>        The SQL expression to apply against the join result.
-                             Ordinarily used to select columns and filter rows from 
-                             the join result. Be sure to select from the "join_result"
-                             table when formulating the SQL expression.
+                             Used to select columns and filter rows AFTER running the join.
+                             Be sure to select from the "join_result" table when formulating
+                             the SQL expression.
                              (e.g. "select c1, c2 as colname from join_result where c2 > 20")
    --datetime-format <fmt>   The datetime format to use writing datetimes.
                              See https://docs.rs/chrono/latest/chrono/format/strftime/index.html
@@ -199,11 +232,12 @@ use std::{
     env,
     fs::File,
     io::{self, BufReader, BufWriter, Read, Write},
+    mem::swap,
     path::{Path, PathBuf},
     str,
 };
 
-use polars::{datatypes::AnyValue, prelude::*, sql::SQLContext};
+use polars::{datatypes::AnyValue, prelude as pl, prelude::*, sql::SQLContext};
 use serde::Deserialize;
 use tempfile::tempdir;
 
@@ -214,44 +248,56 @@ use crate::{
 
 #[derive(Deserialize)]
 struct Args {
-    arg_columns1:          String,
-    arg_input1:            String,
-    arg_columns2:          String,
-    arg_input2:            String,
-    flag_left:             bool,
-    flag_left_anti:        bool,
-    flag_left_semi:        bool,
-    flag_right:            bool,
-    flag_full:             bool,
-    flag_cross:            bool,
-    flag_coalesce:         bool,
-    flag_filter_left:      Option<String>,
-    flag_filter_right:     Option<String>,
-    flag_validate:         Option<String>,
-    flag_nulls:            bool,
-    flag_streaming:        bool,
-    flag_try_parsedates:   bool,
-    flag_decimal_comma:    bool,
-    flag_infer_len:        usize,
-    flag_cache_schema:     bool,
-    flag_low_memory:       bool,
-    flag_no_optimizations: bool,
-    flag_ignore_errors:    bool,
-    flag_asof:             bool,
-    flag_left_by:          Option<String>,
-    flag_right_by:         Option<String>,
-    flag_strategy:         Option<String>,
-    flag_tolerance:        Option<String>,
-    flag_sql_filter:       Option<String>,
-    flag_datetime_format:  Option<String>,
-    flag_date_format:      Option<String>,
-    flag_time_format:      Option<String>,
-    flag_float_precision:  Option<usize>,
-    flag_null_value:       String,
-    flag_output:           Option<String>,
-    flag_delimiter:        Option<Delimiter>,
-    flag_quiet:            bool,
-    flag_ignore_case:      bool,
+    arg_columns1:              String,
+    arg_input1:                String,
+    arg_columns2:              String,
+    arg_input2:                String,
+    flag_left:                 bool,
+    flag_left_anti:            bool,
+    flag_left_semi:            bool,
+    flag_right:                bool,
+    flag_right_anti:           bool,
+    flag_right_semi:           bool,
+    flag_full:                 bool,
+    flag_cross:                bool,
+    flag_non_equi:             Option<String>,
+    flag_coalesce:             bool,
+    flag_filter_left:          Option<String>,
+    flag_filter_right:         Option<String>,
+    flag_validate:             Option<String>,
+    flag_maintain_order:       Option<String>,
+    flag_nulls:                bool,
+    flag_streaming:            bool,
+    flag_try_parsedates:       bool,
+    flag_decimal_comma:        bool,
+    flag_infer_len:            usize,
+    flag_cache_schema:         i8,
+    flag_low_memory:           bool,
+    flag_no_optimizations:     bool,
+    flag_ignore_errors:        bool,
+    flag_asof:                 bool,
+    flag_left_by:              Option<String>,
+    flag_right_by:             Option<String>,
+    flag_strategy:             Option<String>,
+    flag_tolerance:            Option<String>,
+    flag_sql_filter:           Option<String>,
+    flag_datetime_format:      Option<String>,
+    flag_date_format:          Option<String>,
+    flag_time_format:          Option<String>,
+    flag_float_precision:      Option<usize>,
+    flag_null_value:           String,
+    flag_output:               Option<String>,
+    flag_delimiter:            Option<Delimiter>,
+    flag_quiet:                bool,
+    flag_ignore_case:          bool,
+    flag_ignore_leading_zeros: bool,
+}
+
+#[derive(PartialEq, Eq)]
+enum SpecialJoin {
+    NonEqui(String),
+    AsOf,
+    None,
 }
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
@@ -283,44 +329,116 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         s => return fail_incorrectusage_clierror!("Invalid join validation: {s}"),
     };
 
+    let flag_maintain_order = args
+        .flag_maintain_order
+        .unwrap_or_else(|| "none".to_string())
+        .to_lowercase();
+    let maintain_order = match flag_maintain_order.as_str() {
+        "none" => MaintainOrderJoin::None,
+        "left" => MaintainOrderJoin::Left,
+        "right" => MaintainOrderJoin::Right,
+        "left_right" => MaintainOrderJoin::LeftRight,
+        "right_left" => MaintainOrderJoin::RightLeft,
+        s => return fail_incorrectusage_clierror!("Invalid maintain order option: {s}"),
+    };
+
     let join_shape: (usize, usize) = match (
         args.flag_left,
         args.flag_left_anti,
         args.flag_left_semi,
         args.flag_right,
+        args.flag_right_anti,
+        args.flag_right_semi,
         args.flag_full,
         args.flag_cross,
         args.flag_asof,
+        args.flag_non_equi.is_some(),
     ) {
-        (false, false, false, false, false, false, false) => {
-            join.run(JoinType::Inner, validation, false)
+        // default inner join
+        (false, false, false, false, false, false, false, false, false, false) => join.run(
+            JoinType::Inner,
+            validation,
+            maintain_order,
+            SpecialJoin::None,
+        ),
+        // left join
+        (true, false, false, false, false, false, false, false, false, false) => join.run(
+            JoinType::Left,
+            validation,
+            maintain_order,
+            SpecialJoin::None,
+        ),
+        // left anti join
+        (false, true, false, false, false, false, false, false, false, false) => join.run(
+            JoinType::Anti,
+            validation,
+            maintain_order,
+            SpecialJoin::None,
+        ),
+        // left semi join
+        (false, false, true, false, false, false, false, false, false, false) => join.run(
+            JoinType::Semi,
+            validation,
+            maintain_order,
+            SpecialJoin::None,
+        ),
+        // right join
+        (false, false, false, true, false, false, false, false, false, false) => join.run(
+            JoinType::Right,
+            validation,
+            maintain_order,
+            SpecialJoin::None,
+        ),
+        // right anti join
+        // swap left and right data sets and run left anti join
+        (false, false, false, false, true, false, false, false, false, false) => {
+            let mut swapped_join = join;
+            swap(&mut swapped_join.left_lf, &mut swapped_join.right_lf);
+            swap(&mut swapped_join.left_sel, &mut swapped_join.right_sel);
+            swapped_join.run(
+                JoinType::Anti,
+                validation,
+                maintain_order,
+                SpecialJoin::None,
+            )
         },
-        (true, false, false, false, false, false, false) => {
-            join.run(JoinType::Left, validation, false)
+        // right semi join
+        // swap left and right data sets and run left semi join
+        (false, false, false, false, false, true, false, false, false, false) => {
+            let mut swapped_join = join;
+            swap(&mut swapped_join.left_lf, &mut swapped_join.right_lf);
+            swap(&mut swapped_join.left_sel, &mut swapped_join.right_sel);
+            swapped_join.run(
+                JoinType::Semi,
+                validation,
+                maintain_order,
+                SpecialJoin::None,
+            )
         },
-        (false, true, false, false, false, false, false) => {
-            join.run(JoinType::Anti, validation, false)
-        },
-        (false, false, true, false, false, false, false) => {
-            join.run(JoinType::Semi, validation, false)
-        },
-        (false, false, false, true, false, false, false) => {
-            join.run(JoinType::Right, validation, false)
-        },
-        (false, false, false, false, true, false, false) => {
-            join.run(JoinType::Full, validation, false)
-        },
-        (false, false, false, false, false, true, false) => {
-            join.run(JoinType::Cross, validation, false)
-        },
-        (false, false, false, false, false, false, true) => {
+        // full join
+        (false, false, false, false, false, false, true, false, false, false) => join.run(
+            JoinType::Full,
+            validation,
+            maintain_order,
+            SpecialJoin::None,
+        ),
+        // cross join
+        (false, false, false, false, false, false, false, true, false, false) => join.run(
+            JoinType::Cross,
+            validation,
+            MaintainOrderJoin::None,
+            SpecialJoin::None,
+        ),
+
+        // as of join
+        (false, false, false, false, false, false, false, false, true, false) => {
             // safety: flag_strategy is always is_some() as it has a default value
             args.flag_strategy = Some(args.flag_strategy.unwrap().to_lowercase());
             let strategy = match args.flag_strategy.as_deref() {
                 Some("backward") | None => AsofStrategy::Backward,
                 Some("forward") => AsofStrategy::Forward,
                 Some("nearest") => AsofStrategy::Nearest,
-                Some(s) => return fail_clierror!("Invalid asof strategy: {}", s),
+                Some(s) => return fail_incorrectusage_clierror!("Invalid asof strategy: {}", s),
             };
 
             let mut asof_options = AsOfOptions {
@@ -358,7 +476,24 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                         .collect(),
                 );
             }
-            join.run(JoinType::AsOf(asof_options), validation, true)
+            join.run(
+                JoinType::AsOf(asof_options),
+                validation,
+                MaintainOrderJoin::None,
+                SpecialJoin::AsOf,
+            )
+        },
+
+        // non-equi join
+        (false, false, false, false, false, false, false, false, false, true) => {
+            // JoinType::Inner is just a placeholder value to satisfy the compiler
+            // as this is a non-equi join
+            join.run(
+                JoinType::Inner,
+                validation,
+                maintain_order,
+                SpecialJoin::NonEqui(args.flag_non_equi.unwrap()),
+            )
         },
         _ => fail_incorrectusage_clierror!("Please pick exactly one join operation."),
     }?;
@@ -394,7 +529,8 @@ impl JoinStruct {
         mut self,
         jointype: JoinType,
         validation: JoinValidation,
-        asof_join: bool,
+        maintain_order: MaintainOrderJoin,
+        special_join: SpecialJoin,
     ) -> CliResult<(usize, usize)> {
         let mut left_selcols: Vec<_> = self
             .left_sel
@@ -510,7 +646,8 @@ impl JoinStruct {
                 .finish()
                 .collect()?
         } else {
-            if asof_join {
+            if special_join == SpecialJoin::AsOf {
+                // it's an asof join
                 // sort by the asof columns, as asof joins require sorted join column data
                 let left_selcols_vec: Vec<PlSmallStr> =
                     self.left_sel.split(',').map(PlSmallStr::from_str).collect();
@@ -530,18 +667,38 @@ impl JoinStruct {
                     .sort(right_selcols_vec, SortMultipleOptions::default());
             }
 
-            self.left_lf
-                .with_optimizations(optflags)
-                .join_builder()
-                .with(self.right_lf.with_optimizations(optflags))
-                .left_on(left_selcols)
-                .right_on(right_selcols)
-                .how(jointype)
-                .coalesce(coalesce_flag)
-                .allow_parallel(true)
-                .validate(validation)
-                .finish()
-                .collect()?
+            if let SpecialJoin::NonEqui(expr) = special_join {
+                // it's a non-equi join
+                let expr = polars::sql::sql_expr(expr)?;
+
+                // Add "_left" & "_right" suffixes to all columns before doing the non-equi join.
+                // This is necessary as the NonEqui expression is a SQL where clause and the
+                // column names for the left and right data sets are used in the expression.
+                self.left_lf = self.left_lf.select([all().name().suffix("_left")]);
+                self.right_lf = self.right_lf.select([all().name().suffix("_right")]);
+
+                self.left_lf
+                    .with_optimizations(optflags)
+                    .join_builder()
+                    .with(self.right_lf.with_optimizations(optflags))
+                    .join_where(vec![expr])
+                    .collect()?
+            } else {
+                // it's one of the "standard" joins as indicated by jointype
+                self.left_lf
+                    .with_optimizations(optflags)
+                    .join_builder()
+                    .with(self.right_lf.with_optimizations(optflags))
+                    .left_on(left_selcols)
+                    .right_on(right_selcols)
+                    .how(jointype)
+                    .maintain_order(maintain_order)
+                    .coalesce(coalesce_flag)
+                    .allow_parallel(true)
+                    .validate(validation)
+                    .finish()
+                    .collect()?
+            }
         };
 
         let mut results_df = if let Some(sql_filter) = &self.sql_filter {
@@ -601,8 +758,6 @@ impl JoinStruct {
 
 impl Args {
     fn new_join(&mut self, tmpdir: &tempfile::TempDir) -> CliResult<JoinStruct> {
-        // =============== NEW_JOIN HELPER FUNCTIONS =================
-
         // Helper function to create a LazyFrameReader with common settings
         fn create_lazy_reader(
             file_path: &str,
@@ -681,8 +836,31 @@ impl Args {
             Ok(schema)
         }
 
-        // Helper function to setup a LazyFrame with schema handling
-        #[inline]
+        /// Helper function to setup a LazyFrame with schema handling based on cache_schema flag.
+        ///
+        /// # Arguments
+        /// * `input_path` - Path to the input CSV file
+        /// * `comment_char` - Optional comment character to ignore lines starting with it
+        /// * `args` - Command line arguments containing schema caching and other options
+        /// * `delim` - Delimiter character for CSV parsing
+        /// * `debuglog_flag` - Whether debug logging is enabled
+        ///
+        /// # Returns
+        /// Returns a tuple containing:
+        /// * The configured LazyFrame for reading the CSV
+        /// * A boolean indicating if a new schema needs to be created and cached
+        ///
+        /// # Schema Caching Modes
+        /// * `0` - No schema caching, infer schema from data sample using Polars
+        /// * `1` - Cache inferred schema from stats in .pschema.json file
+        /// * `-1` - Use string schema for all columns without caching
+        /// * `-2` - Use string schema for all columns and cache it
+        ///
+        /// # Errors
+        /// Returns error if:
+        /// * File operations fail
+        /// * Schema parsing fails
+        /// * Invalid cache_schema value provided
         fn setup_lazy_frame(
             input_path: &Path,
             comment_char: Option<&PlSmallStr>,
@@ -691,47 +869,96 @@ impl Args {
             debuglog_flag: bool,
         ) -> CliResult<(LazyFrame, bool)> {
             let schema_file = input_path.canonicalize()?.with_extension("pschema.json");
-            let mut create_schema = args.flag_cache_schema;
+            let mut create_schema = false;
+            let cache_schema = if args.flag_infer_len == 0 {
+                0
+            } else {
+                args.flag_cache_schema
+            };
 
             let mut reader =
                 create_lazy_reader(input_path.to_str().unwrap(), comment_char, args, delim);
 
-            if create_schema {
-                let mut valid_schema_exists = schema_file.exists()
-                    && schema_file.metadata()?.modified()? > input_path.metadata()?.modified()?;
+            match cache_schema {
+                0 => {
+                    reader = reader.with_infer_schema_length(if args.flag_infer_len == 0 {
+                        None
+                    } else {
+                        Some(args.flag_infer_len)
+                    });
+                },
+                1 => {
+                    let mut valid_schema_exists = schema_file.exists()
+                        && schema_file.metadata()?.modified()?
+                            > input_path.metadata()?.modified()?;
 
-                if !valid_schema_exists {
-                    let schema = create_schema_from_stats(input_path, args)?;
-                    let stats_schema = Arc::new(schema);
-                    let stats_schema_json = serde_json::to_string_pretty(&stats_schema)?;
+                    if !valid_schema_exists {
+                        let schema = create_schema_from_stats(input_path, args)?;
+                        let stats_schema = Arc::new(schema);
+                        let stats_schema_json = serde_json::to_string_pretty(&stats_schema)?;
 
-                    let mut file = BufWriter::new(File::create(&schema_file)?);
-                    file.write_all(stats_schema_json.as_bytes())?;
-                    file.flush()?;
-                    if debuglog_flag {
-                        log::debug!("Saved schema to file: {}", schema_file.display());
+                        let mut file = BufWriter::new(File::create(&schema_file)?);
+                        file.write_all(stats_schema_json.as_bytes())?;
+                        file.flush()?;
+                        if debuglog_flag {
+                            log::debug!("Saved schema to file: {}", schema_file.display());
+                        }
+                        valid_schema_exists = true;
                     }
-                    valid_schema_exists = true;
-                }
 
-                if valid_schema_exists {
-                    let file = File::open(&schema_file)?;
-                    let mut buf_reader = BufReader::new(file);
-                    let mut schema_json = String::with_capacity(100);
-                    buf_reader.read_to_string(&mut schema_json)?;
-                    let schema: Schema = serde_json::from_str(&schema_json)?;
-                    reader = reader.with_schema(Some(Arc::new(schema)));
+                    if valid_schema_exists {
+                        let file = File::open(&schema_file)?;
+                        let mut buf_reader = BufReader::new(file);
+                        let mut schema_json = String::with_capacity(100);
+                        buf_reader.read_to_string(&mut schema_json)?;
+                        let schema: Schema = serde_json::from_str(&schema_json)?;
+                        reader = reader.with_schema(Some(Arc::new(schema)));
+                        create_schema = false;
+                    } else {
+                        reader = reader.with_infer_schema_length(Some(args.flag_infer_len));
+                        create_schema = true;
+                    }
+                },
+                -1 | -2 => {
+                    // get the headers from the input file
+                    let mut rdr = csv::Reader::from_path(input_path)?;
+                    let csv_fields = rdr.byte_headers()?.clone();
+                    drop(rdr);
+
+                    let mut schema = Schema::with_capacity(csv_fields.len());
+                    for field in &csv_fields {
+                        schema.insert(
+                            PlSmallStr::from_str(simdutf8::basic::from_utf8(field).unwrap()),
+                            polars::datatypes::DataType::String,
+                        );
+                    }
+                    let allstring_schema = Arc::new(schema);
+
+                    reader = reader.with_schema(Some(allstring_schema.clone()));
                     create_schema = false;
-                } else {
-                    reader = reader.with_infer_schema_length(Some(args.flag_infer_len));
-                    create_schema = true;
-                }
-            } else {
-                reader = reader.with_infer_schema_length(if args.flag_infer_len == 0 {
-                    None
-                } else {
-                    Some(args.flag_infer_len)
-                });
+
+                    // create and cache allstring schema
+                    if cache_schema == -2 {
+                        let allstring_schema_json =
+                            serde_json::to_string_pretty(&allstring_schema)?;
+
+                        let mut file = BufWriter::new(File::create(&schema_file)?);
+                        file.write_all(allstring_schema_json.as_bytes())?;
+                        file.flush()?;
+                        if debuglog_flag {
+                            log::debug!(
+                                "Saved allstring_schema to file: {}",
+                                schema_file.display()
+                            );
+                        }
+                    }
+                },
+                _ => {
+                    return fail_incorrectusage_clierror!(
+                        "Invalid --cache-schema value: {cache_schema}. Valid values are 0, 1, -1 \
+                         and -2"
+                    )
+                },
             }
 
             Ok((reader.finish()?, create_schema))
@@ -830,6 +1057,98 @@ impl Args {
             right_lf = right_lf.filter(filter_right_expr);
         }
 
+        if self.flag_ignore_leading_zeros {
+            // Transform the join keys in the left and right dataframe to handle leading zeros
+            // For each join key column:
+            //   1. Cast to string type
+            //   2. If ignore_case is true, convert to lowercase
+            //   3. Trim leading zeros, regardless of the original type
+            // This allows joins to match on values like "001" and "1", "0001ABZ" and "1ABZ".
+            // Note that the output columns will not have leading zeros for the join keys.
+            //
+            // We had to add ignore_case handling to ignore_leading_zeros in the join keys because
+            // the existing code for ignore_case was not working for leading zeros if they are both
+            // used at the same time.
+            // The ignore_case code was creating temporary join key columns while
+            // ignore_leading_zeros was using the existing join key columns.
+            let ignore_case = self.flag_ignore_case;
+            left_lf = left_lf.with_columns(
+                self.arg_columns1
+                    .split(',')
+                    .map(|col| {
+                        let col_name = col.to_string();
+                        pl::col(&col_name).cast(pl::DataType::String).map(
+                            move |s| {
+                                Ok(Some(
+                                    pl::Series::new(
+                                        col_name.clone().into(),
+                                        s.str()?
+                                            .into_iter()
+                                            .map(|x| {
+                                                x.map(|val| {
+                                                    let v = if ignore_case {
+                                                        val.to_lowercase()
+                                                    } else {
+                                                        val.to_string()
+                                                    };
+                                                    if v.starts_with('0') {
+                                                        v.trim_start_matches('0').to_string()
+                                                    } else {
+                                                        v
+                                                    }
+                                                })
+                                                .unwrap_or_default()
+                                            })
+                                            .collect::<Vec<String>>(),
+                                    )
+                                    .into(),
+                                ))
+                            },
+                            pl::GetOutput::from_type(pl::DataType::String),
+                        )
+                    })
+                    .collect::<Vec<_>>(),
+            );
+
+            right_lf = right_lf.with_columns(
+                self.arg_columns2
+                    .split(',')
+                    .map(|col| {
+                        let col_name = col.to_string();
+                        pl::col(&col_name).cast(pl::DataType::String).map(
+                            move |s| {
+                                Ok(Some(
+                                    pl::Series::new(
+                                        col_name.clone().into(),
+                                        s.str()?
+                                            .into_iter()
+                                            .map(|x| {
+                                                x.map(|val| {
+                                                    let v = if ignore_case {
+                                                        val.to_lowercase()
+                                                    } else {
+                                                        val.to_string()
+                                                    };
+                                                    if v.starts_with('0') {
+                                                        v.trim_start_matches('0').to_string()
+                                                    } else {
+                                                        v
+                                                    }
+                                                })
+                                                .unwrap_or_default()
+                                            })
+                                            .collect::<Vec<String>>(),
+                                    )
+                                    .into(),
+                                ))
+                            },
+                            pl::GetOutput::from_type(pl::DataType::String),
+                        )
+                    })
+                    .collect::<Vec<_>>(),
+            );
+        }
+
         Ok(JoinStruct {
             left_lf,
             left_sel: self.arg_columns1.clone(),
