@@ -13,7 +13,7 @@ Returns exitcode 1 when no match is found, unless the '--not-one' flag is used.
 When --quick is enabled, no output is produced and exitcode 0 is returned on 
 the first match.
 
-For examples, see https://github.com/jqnatividad/qsv/blob/master/tests/test_search.rs.
+For examples, see https://github.com/dathere/qsv/blob/master/tests/test_search.rs.
 
 Usage:
     qsv search [options] <regex> [<input>]
@@ -165,7 +165,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
     let mut matches_only = false;
 
-    let flag_flag = args.flag_flag.map_or(false, |column_name| {
+    let flag_flag = args.flag_flag.is_some_and(|column_name| {
         // if --flag column is "M", then we only output the M column
         if column_name == "M" {
             headers.clear();
@@ -197,7 +197,6 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let mut match_ctr: u64 = 0;
     let mut row_ctr: u64 = 0;
     let mut m;
-    let mut buffer = itoa::Buffer::new();
     let invert_match = args.flag_invert_match;
 
     #[allow(unused_assignments)]
@@ -277,7 +276,9 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             if flag_flag {
                 flag_rowi += 1;
                 match_row = if m {
-                    buffer.format(flag_rowi).clone_into(&mut matched_rows);
+                    itoa::Buffer::new()
+                        .format(flag_rowi)
+                        .clone_into(&mut matched_rows);
                     matched_rows.as_bytes()
                 } else {
                     b"0"
@@ -360,7 +361,9 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         if flag_flag {
             flag_rowi += 1;
             match_row = if m {
-                buffer.format(flag_rowi).clone_into(&mut matched_rows);
+                itoa::Buffer::new()
+                    .format(flag_rowi)
+                    .clone_into(&mut matched_rows);
                 matched_rows.as_bytes()
             } else {
                 b"0"
