@@ -395,6 +395,7 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
                 #[cfg(any(feature = "feature_capable", feature = "lite"))]
                 if show_progress {
                     progress.set_style(
+                        #[allow(clippy::literal_string_with_formatting_args)]
                         ProgressStyle::default_bar()
                             .template(
                                 "{msg}\n{spinner:.green} [{elapsed_precise}] \
@@ -812,7 +813,7 @@ async fn sniff_main(mut args: Args) -> CliResult<()> {
         .delimiter(args.flag_delimiter);
     let n_rows = if sfile_info.downloaded_records == 0 {
         //if we have the whole file and not just a sample, we can count the number of rows
-        match util::count_rows(&conf) {
+        match util::count_rows_regular(&conf) {
             Ok(n) => n as usize,
             Err(e) => {
                 cleanup_tempfile(sfile_info.tempfile_flag, tempfile_to_delete)?;
