@@ -63,9 +63,21 @@ template arguments:
     <outdir>                    The directory where the output files will be written.
                                 If it does not exist, it will be created.
                                 If not set, output will be sent to stdout or the specified --output.
+                                When writing to <outdir>, files are organized into subdirectories
+                                of --outsubdir-size (default: 1000) files each to avoid filesystem
+                                navigation & performance issues.
+                                For example, with 3500 records:
+                                  <outdir>/0000/0001.txt through <outdir>/0000/1000.txt
+                                  <outdir>/0001/1001.txt through <outdir>/0001/2000.txt
+                                  <outdir>/0002/2001.txt through <outdir>/0002/3000.txt
+                                  <outdir>/0003/3001.txt through <outdir>/0003/4000.txt
 template options:
     --template <str>            MiniJinja template string to use (alternative to --template-file)
     -t, --template-file <file>  MiniJinja template file to use
+    -j, --globals-json <file>   A JSON file containing global variables to make available in templates.
+                                The JSON properties can be accessed in templates using the "qsv_g"
+                                namespace (e.g. {{qsv_g.school_name}}, {{qsv_g.year}}).
+                                This allows sharing common values across all template renders.
     --outfilename <str>         MiniJinja template string to use to create the filename of the output 
                                 files to write to <outdir>. If set to just QSV_ROWNO, the filestem
                                 is set to the current rowno of the record, padded with leading
@@ -73,6 +85,8 @@ template options:
                                 Note that all the fields, including QSV_ROWNO, are available
                                 when defining the filename template.
                                 [default: QSV_ROWNO]
+    --outsubdir-size <num>      The number of files per subdirectory in <outdir>.
+                                [default: 1000]
     --customfilter-error <msg>  The value to return when a custom filter returns an error.
                                 Use "<empty string>" to return an empty string.
                                 [default: <FILTER_ERROR>]
@@ -106,6 +120,7 @@ Common options:
 use std::{
     fs,
     io::{BufWriter, Write},
+    path::PathBuf,
     sync::{
         atomic::{AtomicBool, AtomicU16, Ordering},
         OnceLock, RwLock,
@@ -121,7 +136,7 @@ use rayon::{
     prelude::IntoParallelRefIterator,
 };
 use serde::Deserialize;
-use simd_json::BorrowedValue;
+use simd_json::{json, BorrowedValue};
 
 use crate::{
     config::{Config, Delimiter, DEFAULT_WTR_BUFFER_CAPACITY},
@@ -137,9 +152,11 @@ struct Args {
     arg_input:               Option<String>,
     arg_outdir:              Option<String>,
     flag_template:           Option<String>,
-    flag_template_file:      Option<String>,
+    flag_template_file:      Option<PathBuf>,
+    flag_globals_json:       Option<PathBuf>,
     flag_output:             Option<String>,
     flag_outfilename:        String,
+    flag_outsubdir_size:     u16,
     flag_customfilter_error: String,
     flag_jobs:               Option<usize>,
     flag_batch:              usize,
@@ -207,6 +224,23 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         Ordering::Relaxed,
     );
 
+    // setup globals JSON context if specified
+    let mut globals_flag = false;
+    let globals_ctx = if let Some(globals_json) = args.flag_globals_json {
+        globals_flag = true;
+        match std::fs::read(globals_json) {
+            Ok(mut bytes) => match simd_json::from_slice(&mut bytes) {
+                Ok(json) => json,
+                Err(e) => return fail_clierror!("Failed to parse globals JSON file: {e}"),
+            },
+            Err(e) => return fail_clierror!("Failed to read globals JSON file: {e}"),
+        }
+    } else {
+        json!("")
+    };
+
+    let globals_ctx_borrowed: simd_json::BorrowedValue = globals_ctx.into();
+
     // Set up minijinja environment
     let mut env = Environment::new();
 
@@ -237,7 +271,14 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         .no_headers(args.flag_no_headers);
     let mut rdr = rconfig.reader()?;
 
-    // read headers
+    // Get width of rowcount for padding leading zeroes and early return
+    let rowcount = util::count_rows(&rconfig)?;
+    if rconfig.no_headers && rowcount == 1 {
+        return Ok(());
+    }
+    let width = rowcount.to_string().len();
+
+    // read headers - the headers are used as MiniJinja variables in the template
     let headers = if args.flag_no_headers {
         csv::StringRecord::new()
     } else {
@@ -277,10 +318,6 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         filename_env.template_from_str("")?
     };
 
-    // Get width of rowcount for padding leading zeroes
-    let rowcount = util::count_rows(&rconfig)?;
-    let width = rowcount.to_string().len();
-
     let mut bulk_wtr = if output_to_dir {
         fs::create_dir_all(args.arg_outdir.as_ref().unwrap())?;
         None
@@ -413,6 +450,13 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 let curr_record = record;
 
                 let mut context = simd_json::borrowed::Object::default();
+                // Add globals context to the record context
+                if globals_flag {
+                    context.insert(
+                        std::borrow::Cow::Borrowed("qsv_g"),
+                        globals_ctx_borrowed.clone(),
+                    );
+                }
                 context.reserve(headers_len);
                 let mut row_number = 0_u64;
 
@@ -438,7 +482,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                         }
                     }
                 } else {
-                    // Use header names
+                    // Use header names as template variables
                     for (header, field) in headers.iter().zip(curr_record.iter()) {
                         context.insert(
                             std::borrow::Cow::Borrowed(header),
@@ -479,11 +523,33 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             .collect_into_vec(&mut batch_results);
 
         let mut outpath = std::path::PathBuf::new();
-        for result_record in &batch_results {
+        let mut current_subdir = None;
+        let outsubdir_numfiles = args.flag_outsubdir_size as usize;
+
+        for (idx, result_record) in batch_results.iter().enumerate() {
             if output_to_dir {
                 // safety: this is safe as output_to_dir = args.arg_outdir.is_some()
                 // and result_record.0 (the filename to use) is_some()
                 outpath.push(args.arg_outdir.as_ref().unwrap());
+
+                // Create subdirectory for every outsubdir_size files
+                // to make it easier to handle & navigate generated files
+                // particularly, if we're using a large input CSV
+                let subdir_num = idx / outsubdir_numfiles;
+
+                if current_subdir == Some(subdir_num) {
+                    outpath.push(format!("{subdir_num:0width$}"));
+                } else {
+                    // Only create new subdir when needed
+                    let subdir_name = format!("{subdir_num:0width$}");
+                    outpath.push(&subdir_name);
+
+                    if !outpath.exists() {
+                        fs::create_dir(&outpath)?;
+                    }
+                    current_subdir = Some(subdir_num);
+                }
+
                 outpath.push(result_record.0.as_deref().unwrap());
 
                 // if output_to_dir is true, we'll be writing a LOT of files (one for each row)
