@@ -64,10 +64,10 @@ impl SelectColumns {
         Ok(Selection(map))
     }
 
-    // commented out because it's unused
-    // pub fn is_empty(&self) -> bool {
-    //     self.selectors.is_empty()
-    // }
+    #[allow(dead_code)]
+    pub fn is_empty(&self) -> bool {
+        self.selectors.is_empty()
+    }
 }
 
 impl fmt::Debug for SelectColumns {
