@@ -61,6 +61,7 @@ pub enum StatsMode {
     FrequencyForceStats,
     #[cfg(feature = "polars")]
     PolarsSchema,
+    Outliers,
     None,
 }
 
@@ -358,55 +359,61 @@ pub fn show_env_vars() {
 
 #[inline]
 pub fn count_rows(conf: &Config) -> Result<u64, CliError> {
+    // Check if ROW_COUNT is already initialized to avoid redundant counting
+    if let Some(count) = ROW_COUNT.get() {
+        return Ok(count.unwrap_or(0));
+    }
+
+    // If not, try using index if available
     if let Some(idx) = conf.indexed().unwrap_or(None) {
-        Ok(idx.count())
-    } else {
-        // index does not exist or is stale,
-        // count records by using polars mem-mapped reader if available
-        // otherwise, count records by iterating through records
-        // Do this only once per invocation and cache the result in ROW_COUNT,
-        // so we don't have to re-count rows every time we need to know the
-        // rowcount for CSVs that don't have an index.
-        #[cfg(feature = "polars")]
-        let count_opt = ROW_COUNT.get_or_init(|| {
-            if let Ok((count, _)) = polars_count_input(conf, false) {
-                Some(count)
-            } else {
-                // if polars_count_input fails, fall back to regular CSV reader
-                if let Ok(mut rdr) = conf.reader() {
-                    let mut count = 0_u64;
-                    let mut _record = csv::ByteRecord::new();
-                    #[allow(clippy::used_underscore_binding)]
-                    while rdr.read_byte_record(&mut _record).unwrap_or_default() {
-                        count += 1;
-                    }
-                    Some(count)
-                } else {
-                    None
-                }
-            }
-        });
+        return Ok(idx.count());
+    }
+    // index does not exist or is stale
 
-        #[cfg(not(feature = "polars"))]
-        let count_opt = ROW_COUNT.get_or_init(|| {
-            if let Ok(mut rdr) = conf.clone().skip_format_check(true).reader() {
-                let mut count = 0_u64;
-                let mut _record = csv::ByteRecord::new();
-                #[allow(clippy::used_underscore_binding)]
-                while rdr.read_byte_record(&mut _record).unwrap_or_default() {
-                    count += 1;
-                }
-                Some(count)
-            } else {
-                None
-            }
-        });
+    // Otherwise, count records by using polars mem-mapped reader if available
+    // If polars is not enabled, count records by iterating through records
+    // Do this only once per invocation and cache the result in ROW_COUNT,
+    // so we don't have to re-count rows every time we need to know the
+    // rowcount for CSVs that don't have an index.
+    ROW_COUNT
+        .get_or_init(|| {
+            // Try different counting methods in order of preference
+            count_rows_with_best_method(conf)
+        })
+        .ok_or_else(|| CliError::Other("Unable to get row count".to_string()))
+}
 
-        match *count_opt {
-            Some(count) => Ok(count),
-            None => Err(CliError::Other("Unable to get row count".to_string())),
+#[cfg(feature = "polars")]
+fn count_rows_with_best_method(conf: &Config) -> Option<u64> {
+    if !conf.no_headers {
+        // Try polars first for files with headers
+        if let Ok(count) = polars_count_input(conf, false) {
+            return Some(count);
         }
     }
+
+    // Fall back to CSV reader
+    count_with_csv_reader(conf)
+}
+
+#[cfg(not(feature = "polars"))]
+fn count_rows_with_best_method(conf: &Config) -> Option<u64> {
+    count_with_csv_reader(conf)
+}
+
+fn count_with_csv_reader(conf: &Config) -> Option<u64> {
+    conf.clone()
+        .skip_format_check(true)
+        .reader()
+        .ok()
+        .map(|mut rdr| {
+            let mut count = 0_u64;
+            let mut record = csv::ByteRecord::new();
+            while rdr.read_byte_record(&mut record).unwrap_or_default() {
+                count += 1;
+            }
+            count
+        })
 }
 
 /// Count rows using "regular" CSV reader
@@ -617,15 +624,21 @@ pub fn mem_file_check(
 
     let conservative_memcheck_work = get_envvar_flag("QSV_MEMORY_CHECK") || conservative_memcheck;
 
+    let mut mem_pct = env::var("QSV_FREEMEMORY_HEADROOM_PCT")
+        .unwrap_or_else(|_| DEFAULT_FREEMEMORY_HEADROOM_PCT.to_string())
+        .parse::<u8>()
+        .unwrap_or(DEFAULT_FREEMEMORY_HEADROOM_PCT);
+
+    // if QSV_FREEMEMORY_HEADROOM_PCT is 0, we skip the memory check
+    if mem_pct == 0 {
+        return Ok(i64::MAX);
+    }
+
     let mut sys = sysinfo::System::new();
     sys.refresh_memory();
     let avail_mem = sys.available_memory();
     let free_swap = sys.free_swap();
     let total_mem = sys.total_memory();
-    let mut mem_pct = env::var("QSV_FREEMEMORY_HEADROOM_PCT")
-        .unwrap_or_else(|_| DEFAULT_FREEMEMORY_HEADROOM_PCT.to_string())
-        .parse::<u8>()
-        .unwrap_or(DEFAULT_FREEMEMORY_HEADROOM_PCT);
 
     // for safety, we don't want to go below 10% memory headroom
     // nor above 90% memory headroom as its too memory-restrictive
@@ -1176,7 +1189,17 @@ pub fn log_end(mut qsv_args: String, now: std::time::Instant) {
     }
 }
 
-// taken from https://gist.github.com/dginev/f6da5e94335d545e0a7b
+/// Truncates a UTF-8 encoded string to a maximum byte length while preserving valid UTF-8 encoding.
+///
+/// This function ensures that the truncation happens at valid UTF-8 character boundaries to avoid
+/// splitting multi-byte characters. It modifies the input string in place.
+///
+/// # Arguments
+///
+/// * `input` - A mutable reference to the String to truncate
+/// * `maxsize` - The maximum desired length in bytes
+///
+/// taken from https://gist.github.com/dginev/f6da5e94335d545e0a7b
 pub fn utf8_truncate(input: &mut String, maxsize: usize) {
     let mut utf8_maxsize = input.len();
     if utf8_maxsize >= maxsize {
@@ -1281,6 +1304,20 @@ impl ColumnNameParser {
 }
 
 #[inline]
+/// Rounds a floating point number to a specified number of decimal places.
+///
+/// This function takes a 64-bit floating point number and rounds it to the specified number of
+/// decimal places using "Bankers Rounding" (Midpoint Nearest Even) strategy. It returns the result
+/// as a String.
+///
+/// # Arguments
+///
+/// * `dec_f64` - The floating point number to round
+/// * `places` - The number of decimal places to round to. If set to 9999, no rounding is performed.
+///
+/// # Returns
+///
+/// * A String containing the rounded number with trailing zeros removed and -0.0 normalized to 0.0
 pub fn round_num(dec_f64: f64, places: u32) -> String {
     use rust_decimal::{Decimal, RoundingStrategy};
 
@@ -1305,6 +1342,23 @@ pub fn round_num(dec_f64: f64, places: u32) -> String {
 }
 
 #[inline]
+/// Transforms a byte slice into a ByteString with optional case-insensitive conversion.
+///
+/// This function takes a byte slice and attempts to convert it to a UTF-8 string. If successful,
+/// it trims whitespace and optionally converts to lowercase. If the input is not valid UTF-8,
+/// it returns the original bytes unchanged.
+///
+/// It's fine-tuned for speed and memory usage, using simdutf8 for UTF-8 validation and
+/// to_lowercase_into for non-allocating, in-place lowercase conversion.
+///
+/// # Arguments
+///
+/// * `bs` - The input byte slice to transform
+/// * `casei` - If true, converts the string to lowercase. If false, leaves case unchanged.
+///
+/// # Returns
+///
+/// * A `ByteString` (Vec<u8>) containing the transformed bytes
 pub fn transform(bs: &[u8], casei: bool) -> ByteString {
     if let Ok(s) = simdutf8::basic::from_utf8(bs) {
         if casei {
@@ -1479,6 +1533,7 @@ pub async fn download_file(
     #[cfg(any(feature = "feature_capable", feature = "lite"))]
     if show_progress {
         pb.set_style(
+            #[allow(clippy::literal_string_with_formatting_args)]
             ProgressStyle::default_bar()
                 .template(if total_size == u64::MAX {
                     // only do a spinner if we don't know the total size
@@ -2066,8 +2121,13 @@ pub fn get_stats_records(
             #[cfg(feature = "polars")]
             StatsMode::PolarsSchema => {
                 // StatsMode::PolarsSchema
-                // we need data types and ranges
-                format!("stats\t{input}\t--infer-boolean\t--stats-jsonl\t--output\t{tempfile_path}")
+                // we need data types, ranges & cardinality
+                format!("stats\t{input}\t--cardinality\t--stats-jsonl\t--output\t{tempfile_path}")
+            },
+            StatsMode::Outliers => {
+                // StatsMode::Outliers
+                // we need data types, ranges, cardinality, quartiles, mad and modes/antimodes
+                format!("stats\t{input}\t--cardinality\t--quartiles\t--mad\t--mode\t--stats-jsonl\t--output\t{tempfile_path}")
             },
             StatsMode::None => unreachable!(), // we returned early on None earlier
         };
@@ -2087,12 +2147,20 @@ pub fn get_stats_records(
         if let Some(jobs) = stats_args.flag_jobs {
             stats_args_str = format!("{stats_args_str}\t--jobs\t{jobs}");
         }
+        if stats_args.flag_nulls {
+            stats_args_str = format!("{stats_args_str}\t--nulls");
+        }
 
         let stats_args_vec: Vec<&str> = stats_args_str.split('\t').collect();
 
         let qsv_bin = std::env::current_exe().unwrap();
         let mut stats_cmd = std::process::Command::new(qsv_bin);
-        stats_cmd.args(stats_args_vec);
+        if mode == StatsMode::Outliers {
+            // set the max length for antimodes
+            stats_cmd.env("QSV_ANTIMODES_LEN", "0").args(stats_args_vec);
+        } else {
+            stats_cmd.args(stats_args_vec);
+        }
         let status = stats_cmd.output()?.status;
         if !status.success() {
             let status_code = status.code();
@@ -2258,6 +2326,7 @@ pub fn optimal_batch_size(rconfig: &Config, batch_size: usize, num_jobs: usize)
 /// Expand the tilde (`~`) from within the provided path.
 /// copied from https://github.com/splurf/simple-expand-tilde
 /// as it was just a small wrapper around simple_home_dir
+#[cfg(not(feature = "lite"))]
 pub fn expand_tilde(path: impl AsRef<Path>) -> Option<PathBuf> {
     let p = path.as_ref();
 
