@@ -77,15 +77,20 @@ fn index_outdated_stats() {
             "max_length",
             "sum_length",
             "avg_length",
+            "stddev_length",
+            "variance_length",
+            "cv_length",
             "mean",
             "sem",
+            "geometric_mean",
+            "harmonic_mean",
             "stddev",
             "variance",
             "cv",
             "nullcount",
             "max_precision",
             "sparsity",
-            "qsv__value",
+            "qsv__value"
         ],
         svec![
             "letter",
@@ -100,6 +105,11 @@ fn index_outdated_stats() {
             "1",
             "3",
             "1",
+            "0",
+            "0",
+            "0",
+            "",
+            "",
             "",
             "",
             "",
@@ -108,7 +118,7 @@ fn index_outdated_stats() {
             "0",
             "",
             "0",
-            "",
+            ""
         ],
         svec![
             "number",
@@ -123,15 +133,20 @@ fn index_outdated_stats() {
             "1",
             "3",
             "1",
+            "",
+            "",
+            "",
             "2",
             "0.4714",
+            "1.8171",
+            "1.6364",
             "0.8165",
             "0.6667",
             "40.8248",
             "0",
             "",
             "0",
-            "",
+            ""
         ],
         svec![
             "qsv__rowcount",
@@ -154,6 +169,11 @@ fn index_outdated_stats() {
             "",
             "",
             "",
+            "",
+            "",
+            "",
+            "",
+            "",
             "3"
         ],
         svec![
@@ -177,6 +197,11 @@ fn index_outdated_stats() {
             "",
             "",
             "",
+            "",
+            "",
+            "",
+            "",
+            "",
             "2"
         ],
         svec![
@@ -200,6 +225,11 @@ fn index_outdated_stats() {
             "",
             "",
             "",
+            "",
+            "",
+            "",
+            "",
+            "",
             "26"
         ],
         svec![
@@ -223,7 +253,12 @@ fn index_outdated_stats() {
             "",
             "",
             "",
-            "597e467c8f605d260295bc4e059ffb683ac06139c701eda3bf6d5df7d6b1bc8f"
+            "",
+            "",
+            "",
+            "",
+            "",
+            "ed38d8c4f2747d82eec243d750fd8448ae4b5ebfb2c85c2323085e3e75b64b6a"
         ],
     ];
 
