@@ -1,7 +1,10 @@
 use crate::{qcheck, workdir::Workdir, Csv, CsvData};
 
 fn prop_reverse(name: &str, rows: CsvData, headers: bool) -> bool {
-    if rows.is_empty() || rows[0].contains(&"\u{FEFF}".to_string()) {
+    if rows.is_empty()
+        || rows[0].contains(&"\u{feff}".to_string())
+        || rows.last().unwrap().contains(&"\u{feff}".to_string())
+    {
         return true;
     }
 
@@ -22,7 +25,10 @@ fn prop_reverse(name: &str, rows: CsvData, headers: bool) -> bool {
         vec![]
     };
     expected.reverse();
-    if expected.is_empty() || expected[0].contains(&"\u{FEFF}".to_string()) {
+    if expected.is_empty()
+        || expected[0].contains(&"\u{feff}".to_string())
+        || expected.last().unwrap().contains(&"\u{feff}".to_string())
+    {
         return true;
     }
     if !headers.is_empty() {
@@ -48,7 +54,10 @@ fn prop_reverse_no_headers() {
 }
 
 fn prop_reverse_indexed(name: &str, rows: CsvData, headers: bool) -> bool {
-    if !rows.is_empty() && rows[0].contains(&"\u{FEFF}".to_string()) {
+    if rows.is_empty()
+        || rows[0].contains(&"\u{feff}".to_string())
+        || rows.last().unwrap().contains(&"\u{feff}".to_string())
+    {
         return true;
     }
 
@@ -69,6 +78,12 @@ fn prop_reverse_indexed(name: &str, rows: CsvData, headers: bool) -> bool {
         vec![]
     };
     expected.reverse();
+    if expected.is_empty()
+        || expected[0].contains(&"\u{feff}".to_string())
+        || expected.last().unwrap().contains(&"\u{feff}".to_string())
+    {
+        return true;
+    }
     if !headers.is_empty() {
         expected.insert(0, headers);
     }
