@@ -120,7 +120,7 @@ fn test_stats<S>(
     S: ::std::ops::Deref<Target = str>,
 {
     let (wrk, mut cmd) = setup(name, rows, headers, use_index, nulls, infer_dates);
-    let field_val = get_field_value(&wrk, &mut cmd, field);
+    let field_val = get_field_value(&wrk, &mut cmd, field).unwrap_or_default();
     // Only compare the first few bytes since floating point arithmetic
     // can mess with exact comparisons.
     // when field = skewness, we're comparing a long sequence of the quartile columns,
@@ -170,7 +170,11 @@ where
     (wrk, cmd)
 }
 
-fn get_field_value(wrk: &Workdir, cmd: &mut process::Command, field: &str) -> String {
+fn get_field_value(
+    wrk: &Workdir,
+    cmd: &mut process::Command,
+    field: &str,
+) -> Result<String, String> {
     if field == "median" {
         cmd.arg("--median");
     }
@@ -188,6 +192,9 @@ fn get_field_value(wrk: &Workdir, cmd: &mut process::Command, field: &str) -> St
     }
 
     let mut rows: Vec<Vec<String>> = wrk.read_stdout(cmd);
+    if rows.is_empty() {
+        return Err(format!("Empty stats for command '{cmd:?}'."));
+    }
     let headers = rows.remove(0);
     let mut sequence: Vec<&str> = vec![];
     for row in &rows {
@@ -200,19 +207,21 @@ fn get_field_value(wrk: &Workdir, cmd: &mut process::Command, field: &str) -> St
                     },
                     "skewness" => {
                         sequence.push(val);
-                        return sequence.join(",");
+                        return Ok(sequence.join(","));
                     },
                     _ => {},
                 },
                 _ => {
                     if &**h == field {
-                        return val.clone();
+                        return Ok(val.clone());
                     }
                 },
             }
         }
     }
-    panic!("BUG: Could not find field '{field}' in headers '{headers:?}' for command '{cmd:?}'.");
+    Err(format!(
+        "Could not find field '{field}' in headers '{headers:?}' for command '{cmd:?}'."
+    ))
 }
 
 stats_tests!(stats_infer_string, "type", &["a"], "String");
@@ -320,25 +329,25 @@ stats_tests!(
     stats_multiple_modes,
     "mode",
     &["a", "a", "b", "b", "c", "d", "e", "e"],
-    "a,b,e,3,1"
+    "a|b|e|3|1"
 );
 stats_tests!(
     stats_multiple_modes_num,
     "mode",
     &["5", "5", "33", "33", "42", "17", "99", "99"],
-    "33,5,99,3,1"
+    "33|5|99|3|1"
 );
 stats_tests!(
     stats_multiple_antimodes,
     "antimode",
     &["a", "a", "b", "b", "c", "d", "e", "e"],
-    "c,d,2,1"
+    "c|d|2|1"
 );
 stats_tests!(
     stats_multiple_antimodes_num,
     "antimode",
     &["5", "5", "33", "33", "42", "17", "98", "99", "99"],
-    "17,42,98,3,1"
+    "17|42|98|3|1"
 );
 stats_tests!(
     stats_range,
@@ -518,7 +527,7 @@ stats_tests!(
     stats_antimode_null,
     "antimode",
     &["", "a", "b", "a"],
-    "NULL,b,2,1"
+    "NULL|b|2|1"
 );
 stats_tests!(stats_median, "median", &["1", "2", "3"], "2");
 stats_tests!(stats_median_null, "median", &["", "1", "2", "3"], "2");
@@ -620,9 +629,11 @@ fn stats_prefer_dmy() {
 
     wrk.create("in2.csv", got);
 
-    // removed variance, stddev, sem & cv columns as its causing flaky CI test for float values
+    // removed stddev_length, variance_length, cv_length, variance, geometric_mean, harmonic_mean,
+    // stddev, sem & cv columns as its causing flaky CI test for float values
     let mut cmd = wrk.command("select");
-    cmd.arg("!/variance|stddev|sem|cv/").arg("in2.csv");
+    cmd.arg("!/stddev_length|variance_length|cv_length|variance|geometric_mean|harmonic_mean|stddev|sem|cv/")
+        .arg("in2.csv");
 
     let got2: String = wrk.stdout(&mut cmd);
     let expected2 = wrk.load_test_resource("boston311-100-stats.csv");
@@ -645,9 +656,11 @@ fn stats_prefer_mdy() {
 
     wrk.create("in2.csv", got);
 
-    // removed variance, stddev, sem & cv columns as its causing flaky CI test for float values
+    // removed stddev_length, variance_length, cv_length, variance, geometric_mean, harmonic_mean,
+    // stddev, sem & cv columns as its causing flaky CI test for float values
     let mut cmd = wrk.command("select");
-    cmd.arg("!/variance|stddev|sem|cv/").arg("in2.csv");
+    cmd.arg("!/stddev_length|variance_length|cv_length|variance|geometric_mean|harmonic_mean|stddev|sem|cv/")
+        .arg("in2.csv");
 
     let got2: String = wrk.stdout(&mut cmd);
 
@@ -670,9 +683,11 @@ fn stats_rounding() {
 
     wrk.create("in2.csv", got);
 
-    // removed variance, stddev, sem & cv columns as its causing flaky CI test for float values
+    // removed stddev_length, variance_length, cv_length, variance, geometric_mean, harmonic_mean,
+    // stddev, sem & cv columns as its causing flaky CI test for float values
     let mut cmd = wrk.command("select");
-    cmd.arg("!/variance|stddev|sem|cv/").arg("in2.csv");
+    cmd.arg("!/stddev_length|variance_length|cv_length|variance|geometric_mean|harmonic_mean|stddev|sem|cv/")
+        .arg("in2.csv");
 
     let got2: String = wrk.stdout(&mut cmd);
     let expected2 = wrk.load_test_resource("boston311-100-everything-8places-stats.csv");
@@ -713,9 +728,11 @@ fn stats_no_date_inference() {
 
     wrk.create("in2.csv", got);
 
-    // removed variance, stddev, sem & cv columns as its causing flaky CI test for float values
+    // removed stddev_length, variance_length, cv_length, variance, geometric_mean, harmonic_mean,
+    // stddev, sem & cv columns as its causing flaky CI test for float values
     let mut cmd = wrk.command("select");
-    cmd.arg("!/variance|stddev|sem|cv/").arg("in2.csv");
+    cmd.arg("!/stddev_length|variance_length|cv_length|variance|geometric_mean|harmonic_mean|stddev|sem|cv/")
+        .arg("in2.csv");
 
     let got2: String = wrk.stdout(&mut cmd);
     let expected2 = wrk.load_test_resource("boston311-100-everything-nodate-stats.csv");
@@ -739,9 +756,11 @@ fn stats_with_date_inference() {
 
     wrk.create("in2.csv", got);
 
-    // removed variance, stddev, sem & cv columns as its causing flaky CI test for float values
+    // removed stddev_length, variance_length, cv_length, variance, geometric_mean, harmonic_mean,
+    // stddev, sem & cv columns as its causing flaky CI test for float values
     let mut cmd = wrk.command("select");
-    cmd.arg("!/variance|stddev|sem|cv/").arg("in2.csv");
+    cmd.arg("!/stddev_length|variance_length|cv_length|variance|geometric_mean|harmonic_mean|stddev|sem|cv/")
+        .arg("in2.csv");
 
     let got2: String = wrk.stdout(&mut cmd);
     let expected2 = wrk.load_test_resource("boston311-100-everything-date-stats.csv");
@@ -761,9 +780,11 @@ fn stats_with_date_inference_default_whitelist() {
 
     wrk.create("in2.csv", got);
 
-    // removed variance, stddev, sem & cv columns as its causing flaky CI test for float values
+    // removed stddev_length, variance_length, cv_length, variance, geometric_mean, harmonic_mean,
+    // stddev, sem & cv columns as its causing flaky CI test for float values
     let mut cmd = wrk.command("select");
-    cmd.arg("!/variance|stddev|sem|cv/").arg("in2.csv");
+    cmd.arg("!/stddev_length|variance_length|cv_length|variance|geometric_mean|harmonic_mean|stddev|sem|cv/")
+        .arg("in2.csv");
 
     let got2: String = wrk.stdout(&mut cmd);
     let expected2 =
@@ -811,9 +832,11 @@ fn stats_with_date_type() {
 
     wrk.create("in2.csv", got);
 
-    // removed variance, stddev, sem & cv columns as its causing flaky CI test for float values
+    // removed stddev_length, variance_length, cv_length, variance, geometric_mean, harmonic_mean,
+    // stddev, sem & cv columns as its causing flaky CI test for float values
     let mut cmd = wrk.command("select");
-    cmd.arg("!/variance|stddev|sem|cv/").arg("in2.csv");
+    cmd.arg("!/stddev_length|variance_length|cv_length|variance|geometric_mean|harmonic_mean|stddev|sem|cv/")
+        .arg("in2.csv");
 
     let got2: String = wrk.stdout(&mut cmd);
     let expected2 = wrk.load_test_resource("boston311-100-everything-datenotime-stats.csv");
@@ -921,7 +944,8 @@ fn stats_cache() {
 
     // removed variance, stddev, sem & cv columns as its causing flaky CI test for float values
     let mut cmd = wrk.command("select");
-    cmd.arg("!/variance|stddev|sem|cv/").arg("in2.csv");
+    cmd.arg("!/variance|geometric_mean|harmonic_mean|stddev|sem|cv/")
+        .arg("in2.csv");
 
     let got2: String = wrk.stdout(&mut cmd);
     let expected2 = wrk.load_test_resource("boston311-100-stats.csv");
@@ -959,7 +983,8 @@ fn stats_cache_negative_threshold() {
 
     // removed variance, stddev, sem & cv columns as its causing flaky CI test for float values
     let mut cmd = wrk.command("select");
-    cmd.arg("!/variance|stddev|sem|cv/").arg("in2.csv");
+    cmd.arg("!/variance|geometric_mean|harmonic_mean|stddev|sem|cv/")
+        .arg("in2.csv");
 
     let got2: String = wrk.stdout(&mut cmd);
     let expected2 = wrk.load_test_resource("boston311-100-stats.csv");
@@ -996,7 +1021,8 @@ fn stats_cache_negative_threshold_unmet() {
 
     // removed variance, stddev, sem & cv columns as its causing flaky CI test for float values
     let mut cmd = wrk.command("select");
-    cmd.arg("!/variance|stddev|sem|cv/").arg("in2.csv");
+    cmd.arg("!/variance|geometric_mean|harmonic_mean|stddev|sem|cv/")
+        .arg("in2.csv");
 
     let got2: String = wrk.stdout(&mut cmd);
     let expected2 = wrk.load_test_resource("boston311-100-stats.csv");
@@ -1036,7 +1062,8 @@ fn stats_cache_negative_threshold_five() {
 
     // removed variance, stddev, sem & cv columns as its causing flaky CI test for float values
     let mut cmd = wrk.command("select");
-    cmd.arg("!/variance|stddev|sem|cv/").arg("in2.csv");
+    cmd.arg("!/variance|geometric_mean|harmonic_mean|stddev|sem|cv/")
+        .arg("in2.csv");
 
     let got2: String = wrk.stdout(&mut cmd);
     let expected2 = wrk.load_test_resource("boston311-100-stats.csv");
@@ -1048,6 +1075,23 @@ fn stats_cache_negative_threshold_five() {
     assert!(!Path::new(&wrk.path("boston311-100.stats.csv.json")).exists());
 }
 
+#[test]
+fn stats_antimodes_len_500() {
+    let wrk = Workdir::new("stats_antimodes_len_500");
+    let test_file = wrk.load_test_file("boston311-100.csv");
+
+    let mut cmd = wrk.command("stats");
+    cmd.env("QSV_ANTIMODES_LEN", "500")
+        .arg("--everything")
+        .arg(test_file);
+
+    let got: String = wrk.stdout(&mut cmd);
+
+    let expected = wrk.load_test_resource("boston311-100-antimodes-len500-stats.csv");
+
+    assert_eq!(dos2unix(&got), dos2unix(&expected).trim_end());
+}
+
 #[test]
 fn stats_infer_boolean_1_0() {
     let wrk = Workdir::new("stats_infer_boolean_1_0");
@@ -1104,9 +1148,11 @@ fn stats_is_ascii() {
 
     wrk.create("in2.csv", got);
 
-    // removed variance, stddev, sem & cv columns as its causing flaky CI test for float values
+    // removed stddev_length, variance_length, cv_length, variance, geometric_mean, harmonic_mean,
+    // stddev, sem & cv columns as its causing flaky CI test for float values
     let mut cmd = wrk.command("select");
-    cmd.arg("!/variance|stddev|sem|cv/").arg("in2.csv");
+    cmd.arg("!/stddev_length|variance_length|cv_length|variance|geometric_mean|harmonic_mean|stddev|sem|cv/")
+        .arg("in2.csv");
 
     let got2: String = wrk.stdout(&mut cmd);
     let expected2 = wrk.load_test_resource("boston311-100-with-nonascii-stats.csv");
@@ -1203,8 +1249,13 @@ fn stats_zero_cv() {
             "max_length",
             "sum_length",
             "avg_length",
+            "stddev_length",
+            "variance_length",
+            "cv_length",
             "mean",
             "sem",
+            "geometric_mean",
+            "harmonic_mean",
             "stddev",
             "variance",
             "cv",
@@ -1226,8 +1277,13 @@ fn stats_zero_cv() {
             "1",
             "5",
             "1",
+            "",
+            "",
+            "",
             "3",
             "0.6325",
+            "2.6052",
+            "2.1898",
             "1.4142",
             "2",
             "47.1405",
@@ -1249,8 +1305,13 @@ fn stats_zero_cv() {
             "3",
             "9",
             "1.8",
+            "",
+            "",
+            "",
             "0",
             "3.1623",
+            "0",
+            "",
             "7.0711",
             "50",
             "",
@@ -1272,8 +1333,13 @@ fn stats_zero_cv() {
             "6",
             "25",
             "5",
+            "",
+            "",
+            "",
             "0",
             "28.8472",
+            "0",
+            "",
             "64.5043",
             "4160.801",
             "",
@@ -1284,7 +1350,8 @@ fn stats_zero_cv() {
         ],
         svec![
             "col4", "Integer", "", "935", "-900", "1000", "1900", "Unsorted", "1", "4", "14",
-            "2.8", "187", "304.3603", "680.5703", "463176", "363.9414", "0", "", "0", ""
+            "2.8", "", "", "", "187", "304.3603", "0", "", "680.5703", "463176", "363.9414", "0",
+            "", "0", ""
         ],
         svec![
             "qsv__rowcount",
@@ -1307,6 +1374,11 @@ fn stats_zero_cv() {
             "",
             "",
             "",
+            "",
+            "",
+            "",
+            "",
+            "",
             "5"
         ],
         svec![
@@ -1330,6 +1402,11 @@ fn stats_zero_cv() {
             "",
             "",
             "",
+            "",
+            "",
+            "",
+            "",
+            "",
             "4"
         ],
         svec![
@@ -1353,6 +1430,11 @@ fn stats_zero_cv() {
             "",
             "",
             "",
+            "",
+            "",
+            "",
+            "",
+            "",
             "93"
         ],
         svec![
@@ -1376,7 +1458,12 @@ fn stats_zero_cv() {
             "",
             "",
             "",
-            "1080eea697a7966a96fcfdcdaee4ae4d1355bce057cae6f27d8bba4684902ba1"
+            "",
+            "",
+            "",
+            "",
+            "",
+            "bfe0b46361bf7532d2ea6fcdc1e2c25c07ad06d15ba448271d5aeea377d2d506"
         ],
     ];
     assert_eq!(got, expected);
@@ -1406,14 +1493,14 @@ fn stats_output_tab_delimited() {
     wrk.assert_success(&mut cmd);
 
     let got = std::fs::read_to_string(out_file).unwrap();
-    let expected = r#"field	type	is_ascii	sum	min	max	range	sort_order	min_length	max_length	sum_length	avg_length	mean	sem	stddev	variance	cv	nullcount	max_precision	sparsity	qsv__value
-col1	Integer		15	1	5	4	Ascending	1	1	5	1	3	0.6325	1.4142	2	47.1405	0		0	
-col2	Integer		10644	0	4321	4321	Descending	1	4	17	3.4	2128.8	685.6979	1533.267	2350907.76	72.0249	0		0	
-col3	String	true		01	10		Ascending	2	2	10	2						0		0	
-qsv__rowcount																				5
-qsv__columncount																				3
-qsv__filesize_bytes																				62
-qsv__fingerprint_hash																				b1d8236344b9e74711338567c4cc54a328cc803762aa2826ff00e9a1924ea407
+    let expected = r#"field	type	is_ascii	sum	min	max	range	sort_order	min_length	max_length	sum_length	avg_length	stddev_length	variance_length	cv_length	mean	sem	geometric_mean	harmonic_mean	stddev	variance	cv	nullcount	max_precision	sparsity	qsv__value
+col1	Integer		15	1	5	4	Ascending	1	1	5	1				3	0.6325	2.6052	2.1898	1.4142	2	47.1405	0		0	
+col2	Integer		10644	0	4321	4321	Descending	1	4	17	3.4				2128.8	685.6979	0		1533.267	2350907.76	72.0249	0		0	
+col3	String	true		01	10		Ascending	2	2	10	2	0	0	0								0		0	
+qsv__rowcount																									5
+qsv__columncount																									3
+qsv__filesize_bytes																									62
+qsv__fingerprint_hash																									14a30758a66a00ca7f90b3b763d16a4195fa7b7427f5ce5afb32ff87aece8d0c
 "#;
     assert_eq!(got, expected);
 }
@@ -1442,14 +1529,14 @@ fn stats_output_ssv_delimited() {
     wrk.assert_success(&mut cmd);
 
     let got = std::fs::read_to_string(out_file).unwrap();
-    let expected = r#"field;type;is_ascii;sum;min;max;range;sort_order;min_length;max_length;sum_length;avg_length;mean;sem;stddev;variance;cv;nullcount;max_precision;sparsity;qsv__value
-col1;Integer;;15;1;5;4;Ascending;1;1;5;1;3;0.6325;1.4142;2;47.1405;0;;0;
-col2;Integer;;10644;0;4321;4321;Descending;1;4;17;3.4;2128.8;685.6979;1533.267;2350907.76;72.0249;0;;0;
-col3;String;true;;01;10;;Ascending;2;2;10;2;;;;;;0;;0;
-qsv__rowcount;;;;;;;;;;;;;;;;;;;;5
-qsv__columncount;;;;;;;;;;;;;;;;;;;;3
-qsv__filesize_bytes;;;;;;;;;;;;;;;;;;;;62
-qsv__fingerprint_hash;;;;;;;;;;;;;;;;;;;;b1d8236344b9e74711338567c4cc54a328cc803762aa2826ff00e9a1924ea407
+    let expected = r#"field;type;is_ascii;sum;min;max;range;sort_order;min_length;max_length;sum_length;avg_length;stddev_length;variance_length;cv_length;mean;sem;geometric_mean;harmonic_mean;stddev;variance;cv;nullcount;max_precision;sparsity;qsv__value
+col1;Integer;;15;1;5;4;Ascending;1;1;5;1;;;;3;0.6325;2.6052;2.1898;1.4142;2;47.1405;0;;0;
+col2;Integer;;10644;0;4321;4321;Descending;1;4;17;3.4;;;;2128.8;685.6979;0;;1533.267;2350907.76;72.0249;0;;0;
+col3;String;true;;01;10;;Ascending;2;2;10;2;0;0;0;;;;;;;;0;;0;
+qsv__rowcount;;;;;;;;;;;;;;;;;;;;;;;;;5
+qsv__columncount;;;;;;;;;;;;;;;;;;;;;;;;;3
+qsv__filesize_bytes;;;;;;;;;;;;;;;;;;;;;;;;;62
+qsv__fingerprint_hash;;;;;;;;;;;;;;;;;;;;;;;;;14a30758a66a00ca7f90b3b763d16a4195fa7b7427f5ce5afb32ff87aece8d0c
 "#;
     assert_eq!(got, expected);
 }
@@ -1481,14 +1568,14 @@ fn stats_output_csvsz_delimited() {
     cmd.arg("decompress").arg(out_file.clone());
 
     let got: String = wrk.stdout(&mut cmd);
-    let expected = r#"field,type,is_ascii,sum,min,max,range,sort_order,min_length,max_length,sum_length,avg_length,mean,sem,stddev,variance,cv,nullcount,max_precision,sparsity,qsv__value
-col1,Integer,,15,1,5,4,Ascending,1,1,5,1,3,0.6325,1.4142,2,47.1405,0,,0,
-col2,Integer,,10644,0,4321,4321,Descending,1,4,17,3.4,2128.8,685.6979,1533.267,2350907.76,72.0249,0,,0,
-col3,String,true,,01,10,,Ascending,2,2,10,2,,,,,,0,,0,
-qsv__rowcount,,,,,,,,,,,,,,,,,,,,5
-qsv__columncount,,,,,,,,,,,,,,,,,,,,3
-qsv__filesize_bytes,,,,,,,,,,,,,,,,,,,,62
-qsv__fingerprint_hash,,,,,,,,,,,,,,,,,,,,b1d8236344b9e74711338567c4cc54a328cc803762aa2826ff00e9a1924ea407"#;
+    let expected = r#"field,type,is_ascii,sum,min,max,range,sort_order,min_length,max_length,sum_length,avg_length,stddev_length,variance_length,cv_length,mean,sem,geometric_mean,harmonic_mean,stddev,variance,cv,nullcount,max_precision,sparsity,qsv__value
+col1,Integer,,15,1,5,4,Ascending,1,1,5,1,,,,3,0.6325,2.6052,2.1898,1.4142,2,47.1405,0,,0,
+col2,Integer,,10644,0,4321,4321,Descending,1,4,17,3.4,,,,2128.8,685.6979,0,,1533.267,2350907.76,72.0249,0,,0,
+col3,String,true,,01,10,,Ascending,2,2,10,2,0,0,0,,,,,,,,0,,0,
+qsv__rowcount,,,,,,,,,,,,,,,,,,,,,,,,,5
+qsv__columncount,,,,,,,,,,,,,,,,,,,,,,,,,3
+qsv__filesize_bytes,,,,,,,,,,,,,,,,,,,,,,,,,62
+qsv__fingerprint_hash,,,,,,,,,,,,,,,,,,,,,,,,,14a30758a66a00ca7f90b3b763d16a4195fa7b7427f5ce5afb32ff87aece8d0c"#;
     assert_eq!(got, expected);
 }
 
