@@ -300,6 +300,10 @@ fn validate_adur_public_toilets_dataset_with_json_schema_valid_output() {
         .arg("schema.json")
         .args(["--valid-output", "-"]);
 
+    let out = wrk.output_stderr(&mut cmd);
+    let expected = "13\n";
+    assert_eq!(out, expected);
+
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
     let expected = vec![
         svec!["ExtractDate", "OrganisationURI", "OrganisationLabel", "ServiceTypeURI", "ServiceTypeLabel", "LocationText", "CoordinateReferenceSystem", "GeoX", "GeoY", "GeoPointLicensingURL", "Category", "AccessibleCategory", "RADARKeyNeeded", "BabyChange", "FamilyToilet", "ChangingPlace", "AutomaticPublicConvenience", "FullTimeStaffing", "PartOfCommunityScheme", "CommunitySchemeName", "ChargeAmount", "InfoURL", "OpeningHours", "ManagedBy", "ReportEmail", "ReportTel", "Notes", "UPRN", "Postcode", "StreetAddress", "GeoAreaURI", "GeoAreaLabel"], 
@@ -319,10 +323,6 @@ fn validate_adur_public_toilets_dataset_with_json_schema_valid_output() {
     ];
     assert_eq!(got, expected);
 
-    let num_recs = wrk.output_stderr(&mut cmd);
-    let num_recs_expected = "13".to_string();
-    assert_eq!(num_recs.trim_end(), num_recs_expected);
-
     wrk.assert_err(&mut cmd);
 }
 
